// -*- C++ -*-
/*!
 * @file  Edge.cpp
 * @brief Edge Component
 * $Date$
 *
 * $Id$
 */

#include "Edge.h"
#include "Camera.h"

IplImage* src = NULL;
IplImage* gray = NULL;
IplImage* edge = NULL;
IplImage* img = NULL;

int i = 0;

// Module specification
// <rtc-template block="module_spec">
static const char* edge_spec[] =
  {
    "implementation_id", "Edge",
    "type_name",         "Edge",
    "description",       "Edge Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_th", "3",


    ""
  };
// </rtc-template>

Edge::Edge(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_ImageDataIn("ImageData", m_ImageData),
    m_ImageDataOut("ImageData", m_ImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("ImageData", m_ImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("ImageData", m_ImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

Edge::~Edge()
{
}


RTC::ReturnCode_t Edge::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_th", m_int_th, "3");

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t Edge::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Edge::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Edge::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Edge::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Edge::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Edge::onExecute(RTC::UniqueId ec_id)
{
  int thre;
  m_ImageDataIn.read();

  src = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 3);
  gray = cvCreateImage(cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  edge = cvCreateImage(cvSize(m_int_width,m_int_height),IPL_DEPTH_16S, 1);
  img = cvCreateImage(cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);

  for(i=0;i<m_ImageData.data.length();i++){
    src->imageData[i]=m_ImageData.data[i];
    //printf("test\n");
  }


  cvCvtColor(src, gray, CV_RGB2GRAY);
  
  thre=m_int_th;
  if(thre>5)thre=5;
  else if(thre<0)thre=0;
  cvLaplace(gray,edge,thre);//3);

  
  for(i = 0;i < img->imageSize;i++){
    img->imageData[i] = (unsigned char)(((short*)edge->imageData)[i]/8+128);
  }
  
  //cvThreshold(edge,img,50.0,255.0,CV_THRESH_BINARY);

  for(i = 0;i < src->imageSize;i++){
    src->imageData[i] = img->imageData[i/3];
  }
  
  m_ImageData.data.length(src->imageSize);
  memcpy((void *)&(m_ImageData.data[0]),src->imageData,src->imageSize);

  /*
  for(int i=0;i<src->imageSize;i++){
    m_ImageData.data[i]=src->imageData[i];
  }
  */

  //m_ImageData.data[0]=src->imageData[0];
 
  m_ImageDataOut.write();

  cvReleaseImage(&src);
  cvReleaseImage(&gray);
  cvReleaseImage(&edge);
  cvReleaseImage(&img);

  cvWaitKey(100);
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Edge::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Edge::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Edge::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Edge::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Edge::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void EdgeInit(RTC::Manager* manager)
  {
    RTC::Properties profile(edge_spec);
    manager->registerFactory(profile,
                             RTC::Create<Edge>,
                             RTC::Delete<Edge>);
  }
  
};


