// -*- C++ -*-
/*!
 * @file  CaribComponent.cpp
 * @brief Carib Component
 * $Date$
 *
 * $Id$
 */

#include "CaribComponent.h"
#include "Camera.h"


IplImage* src =NULL;// = cvCreateImage( cvSize(320,240),IPL_DEPTH_8U, 1);
IplImage* osrc=NULL;// = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;
IplImage* odst =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );
IplImage* dst =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );
IplImage* cdst =NULL;// cvCreateImage( cvGetSize(src), 8, 3 );

CvPoint textPos;
CvFont textFont;
CvScalar color;
char *stateStr = new char[128];

int phase=0;

// Module specification
// <rtc-template block="module_spec">
static const char* caribcomponent_spec[] =
  {
    "implementation_id", "CaribComponent",
    "type_name",         "CaribComponent",
    "description",       "Carib Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_param1", "8",
    "conf.default.int_param2", "7",
    "conf.default.int_param3", "0",

    ""
  };
// </rtc-template>

CaribComponent::CaribComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

CaribComponent::~CaribComponent()
{
}


RTC::ReturnCode_t CaribComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_param1", m_int_param1, "9");
  bindParameter("int_param2", m_int_param2, "9");
  bindParameter("int_param3", m_int_param3, "0");
  
  cvInitFont(&textFont,CV_FONT_VECTOR0, 0.4f, 0.7f, 0.0f,2);
  color.val[0]=0; color.val[1]=0; color.val[2]=0xff; color.val[3]=0;
  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t CaribComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CaribComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CaribComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CaribComponent::onActivated(RTC::UniqueId ec_id)
{
  src = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  osrc = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;

  odst = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);

  dst = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t CaribComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&osrc);
  cvReleaseImage(&src);
  cvReleaseImage(&odst);
  cvReleaseImage(&dst);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t CaribComponent::onExecute(RTC::UniqueId ec_id)
{

  textPos.x=3;
  textPos.y=20;

  /*inport part*/
  m_InImageDataIn.read();
  if(m_InImageData.data.length()==m_int_width*m_int_height*3){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);

 
    switch(phase)
      {
      case 0:
	stateStr="test 0: OK";
	break;
      case 1:
	stateStr="test 1: OK";
	break;
      case 2:
	stateStr="test 2: OK";
	break;
      case 3:
	stateStr="test 3: OK";
	break;
      case 4:
	stateStr="test 4: OK";
	break;
      }
    

    cvPutText(osrc,stateStr,textPos,&textFont,color);
    
    
    /*outport part begin*/
    m_OutImageData.data.length(m_int_width*m_int_height*3);
    memcpy((void *)&(m_OutImageData.data[0]),osrc->imageData,m_int_width*m_int_height*3);
    m_OutImageDataOut.write();
    /*outport part end*/
    
  }
  
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t CaribComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CaribComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CaribComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CaribComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CaribComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CaribComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(caribcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<CaribComponent>,
                             RTC::Delete<CaribComponent>);
  }
  
};


