// -*- C++ -*-
/*!
 * @file  ErodeComponent.cpp
 * @brief Erode Component
 * $Date$
 *
 * $Id$
 */

#include "ErodeComponent.h"
#include "Camera.h"
IplImage* osrc = NULL;
IplImage* dst = NULL;

int times;

// Module specification
// <rtc-template block="module_spec">
static const char* erodecomponent_spec[] =
  {
    "implementation_id", "ErodeComponent",
    "type_name",         "ErodeComponent",
    "description",       "Erode Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_param", "1",

    ""
  };
// </rtc-template>

ErodeComponent::ErodeComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

ErodeComponent::~ErodeComponent()
{
}


RTC::ReturnCode_t ErodeComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_param", m_int_param, "1");
  
  times=m_int_param;

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t ErodeComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ErodeComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ErodeComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ErodeComponent::onActivated(RTC::UniqueId ec_id)
{
  osrc = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 3);
  dst = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 3);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t ErodeComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&osrc);
  cvReleaseImage(&dst);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t ErodeComponent::onExecute(RTC::UniqueId ec_id)
{

  /*inport part*/
  m_InImageDataIn.read();
  if(m_InImageData.data.length()==m_int_width*m_int_height*3){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);

    times=m_int_param;
    if(times<1)
      times=1;
    cvErode(osrc,dst,NULL,times);
 
    /*outport part begin*/
    m_OutImageData.data.length(m_int_width*m_int_height*3);
    memcpy((void *)&(m_OutImageData.data[0]),dst->imageData,m_int_width*m_int_height*3);
    m_OutImageDataOut.write();
    /*outport part end*/
  }


  cvWaitKey(100);


  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t ErodeComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ErodeComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ErodeComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ErodeComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ErodeComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ErodeComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(erodecomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<ErodeComponent>,
                             RTC::Delete<ErodeComponent>);
  }
  
};


