// -*- C++ -*-
/*!
 * @file  GrayComponent.cpp
 * @brief Gray Component
 * $Date$
 *
 * $Id$
 */

#include "GrayComponent.h"
#include "Camera.h"


IplImage* src =NULL;// = cvCreateImage( cvSize(320,240),IPL_DEPTH_8U, 1);
IplImage* osrc=NULL;// = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;

// Module specification
// <rtc-template block="module_spec">
static const char* graycomponent_spec[] =
  {
    "implementation_id", "GrayComponent",
    "type_name",         "GrayComponent",
    "description",       "Gray Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.m_int_width", "320",
    "conf.default.m_int_height", "240",

    ""
  };
// </rtc-template>

GrayComponent::GrayComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_ImageDataOut("ImageData", m_ImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("ImageData", m_ImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

GrayComponent::~GrayComponent()
{
}


RTC::ReturnCode_t GrayComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  
  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t GrayComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GrayComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GrayComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t GrayComponent::onActivated(RTC::UniqueId ec_id)
{
  src = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  osrc = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t GrayComponent::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t GrayComponent::onExecute(RTC::UniqueId ec_id)
{
  /*inport part begin*/
  m_InputImageIn.read();

  for(i=0;i<m_InputImage.data.length();i++){
    osrc->imageData[i]=m_InputImage.data[i];
  }
  /*inport part end*/
  
  cvCvtColor( osrc, src, CV_BGR2GRAY );
  cvCvtColor( dst, osrc, CV_GRAY2BGR );

  /*outport part begin*/
  m_OutputImage.data.length(osrc->imageSize);
  for(int i=0;i<osrc->imageSize;i++){
    //for(int i=0;i<color_dst->imageSize;i++){
    //m_OutputImage.data[i]=color_dst->imageData[i];
    m_OutputImage.data[i]=osrc->imageData[i];
  }
  m_OutputImageOut.write();
  /*outport part end*/


  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t GrayComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GrayComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GrayComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GrayComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GrayComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void GrayComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(graycomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<GrayComponent>,
                             RTC::Delete<GrayComponent>);
  }
  
};


