// -*- C++ -*-
/*!
 * @file  SbackComponent.cpp
 * @brief Static Background Component
 * $Date$
 *
 * $Id$
 */

#include "SbackComponent.h"
#include "Camera.h"

IplImage* src =NULL;// = cvCreateImage( cvSize(320,240),IPL_DEPTH_8U, 1);
IplImage* osrc=NULL;// = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;
IplImage* dst =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );
IplImage* cdst =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );
IplImage* ddst =NULL;// cvCreateImage( cvGetSize(src), 8, 3 );
IplImage* odst =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );
IplImage* mask =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );

// Module specification
// <rtc-template block="module_spec">
static const char* sbackcomponent_spec[] =
  {
    "implementation_id", "SbackComponent",
    "type_name",         "SbackComponent",
    "description",       "Static Background Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_type", "0",
    "conf.default.int_th", "10",

    ""
  };
// </rtc-template>

SbackComponent::SbackComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

SbackComponent::~SbackComponent()
{
}


RTC::ReturnCode_t SbackComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_type", m_int_type, "0");
  bindParameter("int_th", m_int_th, "10");
 
  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t SbackComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SbackComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SbackComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t SbackComponent::onActivated(RTC::UniqueId ec_id)
{
  int i;

  src = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  osrc = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;

  mask = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  dst= cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  odst= cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  cdst = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;
 
  /*inport part begin*/
  m_InImageDataIn.read();
  
  for(i=0;i<m_InImageData.data.length();i++){
    osrc->imageData[i]=m_InImageData.data[i];
  }
  /*inport part end*/

  cvCvtColor(osrc,src,CV_BGR2GRAY);
  cvCopy(src,odst);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t SbackComponent::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}



RTC::ReturnCode_t SbackComponent::onExecute(RTC::UniqueId ec_id)
{
  int i;

  /*inport part begin*/
  m_InImageDataIn.read();
  
  /*
  for(i=0;i<m_InImageData.data.length();i++){
    osrc->imageData[i]=m_InImageData.data[i];
  }
  */

  //memcpy((void *)&(m_InImageData.data[0),osrc->imageData,m_int_width*m_int_height*3);//m_InImageData.data.length());
  if(m_InImageData.data.length()>0){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);//m_InImageData.data.length());
  }

  /*inport part end*/

  cvCvtColor(osrc,src,CV_BGR2GRAY);

  for(int i=0;i<m_int_width*m_int_height;i++){
    unsigned char new_g=*(unsigned char*)&src->imageData[i];
    unsigned char bg_g=*(unsigned char*)&odst->imageData[i];
 
    int diff =abs( (int)new_g -(int)bg_g);
    if(diff>m_int_th){
      if(m_int_type==0)//color
	mask->imageData[i]=0;
      else if(m_int_type==1)//gray
	mask->imageData[i]=1;
      else if(m_int_type==2)//black-white
	mask->imageData[i]=0;
    }else{
      if(m_int_type==0)
	mask->imageData[i]=1;
      else if(m_int_type==1)//gray
      	mask->imageData[i]=0;
      else if(m_int_type==2)//black-white
	mask->imageData[i]=1;
    }
  }

  switch(m_int_type)
    {
    case 0:
      /*color*/
      memset(cdst->imageData, 0, m_int_width*m_int_height*3);
      cvCopy(cdst,osrc,mask);
      break;
    case 1:
      /*gray*/
      memset(dst->imageData, 0, m_int_width*m_int_height);
      cvCopy(src,dst,mask);
      cvCvtColor( dst, osrc, CV_GRAY2BGR );
      break;
    case 2:
      //black-white
      memset(cdst->imageData, 0, m_int_width*m_int_height*3);
      memset(osrc->imageData, 255, m_int_width*m_int_height*3);
      cvCopy(cdst,osrc,mask);
      break;
    }

   /*outport part begin*/
  m_OutImageData.data.length(osrc->imageSize);
  memcpy((void *)&(m_OutImageData.data[0]),osrc->imageData,m_int_width*m_int_height*3);//m_InImageData.data.length());
  m_OutImageDataOut.write();
  /*outport part end*/


  cvWaitKey(100);



  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t SbackComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SbackComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SbackComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SbackComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SbackComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void SbackComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(sbackcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<SbackComponent>,
                             RTC::Delete<SbackComponent>);
  }
  
};


